/* -*- C++ -*-
 * 
 * Bertrand Leconte <B.Leconte@mail.dotcom.fr>
 */

#ifndef __CommandDlg_h__
#define __CommandDlg_h__

#include <kapp.h>
#include <qwidget.h>
#include <qlabel.h>
#include <qlined.h>
#include <qpushbt.h>
#include <qsocknot.h>
#include <qlayout.h>

#include "mymlined.h"
#include "kprocess.h"


/**
 * Implements the inside of the tabCtrl with mainly
 * three items :
 *  - a LineEdit to type in a hostname
 *  - a Go/Stop button
 *  - a MultiLineEdit to receive the output
 */
class CommandDlg: public QWidget
{
    Q_OBJECT;

public:
    CommandDlg(QWidget* parent = NULL, const char* name = NULL);
    virtual  ~CommandDlg();
    void      setCommand(const char *);

    // callback to inform us that we are shown
    void      tabSelected();

    // callback to inform us that we have to kill the command
    void      tabDeselected();

protected:
  //void          resizeEvent(QResizeEvent *);
  virtual bool  checkInput(QString *);
  virtual void  buildCommandLine(QString);
  
  // Widgets
  QLabel*          commandLbl1;
  QLineEdit*       commandArgs;
  QPushButton*     commandGoBtn;
  QPushButton*     commandStopBtn;
  MyMultiLineEdit* commandTextArea;
  
  // Layout
  QBoxLayout*      layout1;	// Vertical
  QBoxLayout*      layout2;	// Horizontal
  
    bool             isGoBtnEnabled;
    // our child
    KProcess childProcess;

public slots:
    void slotCmdStdout();
//void slotProcessDead(int);
    void slotProcessDead(KProcess *);

protected slots:
    void slotEnableGoBtn(const char *);
    void slotStopCommand();
    void slotLauchCommand();

private:
    // socket notifier for Qt
    QSocketNotifier *sn;
    
};
#endif // __CommandDlg_h__



