/*
 * HostDlg - dialog for the host program
 *
 * This is a part of KDE network utilities
 *
 * Bertrand Leconte <B.Leconte@mail.dotcom.fr>
 *
 * This program is free software (GPL). See the file COPYING for details.
 */

#include "HostDlg.h"
#include "mHostDlg.cpp"

#include <qregexp.h>
//#include <qfontmet.h>

// This is the unit used to separate widgets
#define SEPARATION 10

/*
 * Constructor
 */
HostDlg::HostDlg(QWidget* parent, const char* name)
  : CommandDlg(parent, name)
{
  delete layout1;
  delete layout2;

  layout1 = new QBoxLayout(this, QBoxLayout::TopToBottom, SEPARATION);
  CHECK_PTR(layout1);

  layout2 = new QBoxLayout(QBoxLayout::LeftToRight, SEPARATION);
  CHECK_PTR(layout2);
  layout1->addLayout(layout2, 0);

  // Frame for options
  frame1 = new QFrame(this, "frame_1");
  CHECK_PTR(frame1);
  frame1->setFrameStyle(QFrame::Box | QFrame::Sunken);
  layout1->addWidget(frame1, 0);

  layout3 = new QBoxLayout(frame1, QBoxLayout::LeftToRight, SEPARATION/2);
  CHECK_PTR(layout3);

  // Re-make the layout of CommandDlg
  layout2->addWidget(commandLbl1);
  layout2->addWidget(commandArgs);
  layout2->addSpacing(2*SEPARATION);
  layout2->addWidget(commandGoBtn);
  layout2->addWidget(commandStopBtn);

  // Layout of options
  layout3->addStretch(10);


  hostLbl1 = new QLabel("Search for:", frame1, "Label_1");
  CHECK_PTR(hostLbl1);
  hostLbl1->adjustSize();
  hostLbl1->setFixedSize(hostLbl1->size());
  layout3->addWidget(hostLbl1);

  hostCb1 = new QComboBox(frame1, "ComboBox_1");
  CHECK_PTR(hostCb1);
  // This is the array of what is the search for ComboBox.
  // Warning: member buildCommandLine use index=1 for PTR and index=0 for A
  static const char *queryTypes[] 
    = { "address (A)", "name (PTR)", "name server (NS)", 
	"mail exchanger (MX)", "alias (CNAME)", 
	"start of authority (SOA)", "any record (ANY)", 0};
  hostCb1->insertStrList(queryTypes);
  hostCb1->adjustSize();
  hostCb1->setFixedSize(hostCb1->size());
  layout3->addWidget(hostCb1);

  layout3->addStretch(10);

  hostLbl2 = new QLabel("Server:", frame1, "Label_2");
  CHECK_PTR(hostLbl2);
  hostLbl2->adjustSize();
  hostLbl2->setFixedSize(hostLbl2->size());
  //hostLbl2->resize(hostLbl2->width(), cbh);
  layout3->addWidget(hostLbl2);

  hostCb2 = new QComboBox(TRUE, frame1, "ComboBox_2");
  CHECK_PTR(hostCb2);
  hostCb2->insertItem("111.111.111.111");
  hostCb2->adjustSize();
  hostCb2->setFixedSize(hostCb2->size());
  hostCb2->changeItem("default server", 0);
  // we should read the /etc/resolv.conf file
  hostCb2->setInsertionPolicy(QComboBox::AtTop);
  layout3->addWidget(hostCb2);

  layout3->addStretch(10);
  layout3->activate();

  layout1->addWidget(commandTextArea, 10);
  layout1->activate();
 
}

/*
 * Destructor
 */
HostDlg::~HostDlg()
{
}


/**
 * build the command line from widgets
 */
void
HostDlg::buildCommandLine(QString args)
{
  QString s, queryType;
  QRegExp *regexp;
  int len, start;
  
  childProcess.wantCommunication(TRUE);
  childProcess.clearArguments();

  // search for: combobox
  s = ((QString)(hostCb1->currentText())).stripWhiteSpace();;
  // - extract the query type from the string
  regexp = new QRegExp("([A-Za-z]+)");
  CHECK_PTR(regexp);
  start = regexp->match(s, 0, &len);
  ASSERT(start>0);
  queryType = s.mid(start+1, len-2); // to remove ()
  //debug ("queryType=%s", (const char *)queryType);
  delete regexp;
  if (!strcmp(queryType, "A")) {
    // - if q=A and the args is an IP address, then q=PTR
    if (args.contains(QRegExp("^[0-9]+\\.[0-9]+\\.[0-9]+\\.[0-9]+$")) > 0) {
      //debug("This is an IP address");
      queryType = "PTR";
      hostCb1->setCurrentItem(1);
    }
  } else if (!strcmp(queryType, "PTR")) {
    // - if q=PTR and the args is an name, then q=A
    if (args.contains(QRegExp("^[0-9]+\\.[0-9]+\\.[0-9]+\\.[0-9]+$")) > 0) {
    } else {
      //debug("This is _not_ an IP address");
      queryType = "A";
      hostCb1->setCurrentItem(0);
    }
    
  }
  childProcess << "-t" << queryType;
  childProcess << args;

  // server: combobox
  if (strcmp("default server", hostCb2->currentText())) {
    // it's not the default server
    childProcess << hostCb2->currentText();
  }
}
