/*
 * TracerouteDlg - dialog for the traceroute program
 *
 * This is a part of KDE network utilities
 *
 * Bertrand Leconte <B.Leconte@mail.dotcom.fr>
 *
 * This program is free software (GPL). See the file COPYING for details.
 */

#include "TracerouteDlg.h"
#include "mTracerouteDlg.cpp"


// This is the unit used to separate widgets
#define SEPARATION 10

/*
 * Constructor
 */
TracerouteDlg::TracerouteDlg(QWidget* parent, const char* name)
  : CommandDlg(parent, name)
{
  delete layout1;
  //  delete layout2;

  layout1 = new QBoxLayout(this, QBoxLayout::TopToBottom, SEPARATION);
  CHECK_PTR(layout1);

  layout2 = new QBoxLayout(QBoxLayout::LeftToRight, SEPARATION);
  CHECK_PTR(layout2);
  layout1->addLayout(layout2, 0);

  // Frame for options
  frame1 = new QFrame(this, "frame_1");
  CHECK_PTR(frame1);
  frame1->setFrameStyle(QFrame::Box | QFrame::Sunken);
  layout1->addWidget(frame1, 0);

  layout3 = new QBoxLayout(frame1, QBoxLayout::LeftToRight, SEPARATION/2);
  CHECK_PTR(layout3);

  // Re-make the layout of CommandDlg
  layout2->addWidget(commandLbl1);
  layout2->addWidget(commandArgs);
  layout2->addSpacing(2*SEPARATION);
  layout2->addWidget(commandGoBtn);
  layout2->addWidget(commandStopBtn);

  // Layout of options
  layout3->addStretch(10);

  tracerouteCb1 = new QCheckBox("Host name resolution", frame1, "cb_1");
  tracerouteCb1->setChecked(TRUE);
  tracerouteCb1->adjustSize();
  tracerouteCb1->setFixedSize(tracerouteCb1->size());
  layout3->addWidget(tracerouteCb1, 0);

  layout3->addStretch(10);

  tracerouteLbl2 = new QLabel("Maximum number of hops:", frame1, "Label_2"); 
  tracerouteLbl2->adjustSize();
  tracerouteLbl2->setFixedSize(tracerouteLbl2->size());
  layout3->addWidget(tracerouteLbl2, 0);

  tracerouteLe2 = new QLineEdit(frame1, "LineEdit_2");
  tracerouteLe2->setMaxLength(3);
  tracerouteLe2->setText("30");
  //tracerouteLe2->adjustSize();
  tracerouteLe2->setFixedSize(fontMetrics().width("0000"), 
			      (int)(1.5*fontMetrics().height()));
			      //tracerouteLe2->height());
  layout3->addWidget(tracerouteLe2, 0);

  layout3->addStretch(10);
  layout3->activate();

  layout1->addWidget(commandTextArea, 10);
  layout1->activate();
}

/*
 * Destructor
 */
TracerouteDlg::~TracerouteDlg()
{
}

/**
 * build the command line from widgets
 */
void
TracerouteDlg::buildCommandLine(QString args)
{
  int     iMaxHops;
  QString sMaxHops;

  // Check the Maximum hop count
  iMaxHops = atoi(tracerouteLe2->text());
  if (iMaxHops <=0) {
    iMaxHops = 30;
  }
  sMaxHops.sprintf("%d", iMaxHops);
  tracerouteLe2->setText(sMaxHops);
    
  childProcess.wantCommunication(TRUE);
  childProcess.clearArguments();
  if (!tracerouteCb1->isChecked()) {
    childProcess << "-n";
  }
  childProcess << "-m" << sMaxHops;
  childProcess << (const char *)args;
}




