#ifndef KMLINED_H
#define KMLINED_H

/*
 kmlined.h

 Requires QT 1.2+ from www.troll.no and kdelib 0.7.4+ from www.kde.org.

 This is the header file for the KMultiLineEdit class. It is derived from
 QMultiLineEdit but adds some functionality that would be good to have in
 all KDE programs using a line edit control.
 It adds the short cuts CNTRL-INS which copies the selected text to
 the clipboard and SHIFT-INS which copies text from the clipboard to
 the line edit.
 It also adds a popup menu controlled by the right mouse button which
 gives you the four options: Copy, Paste, Cut and Delete.
 If you wan't to add or change items in the menu you can retrieve it
 with KMultiLineEdit::getMenu().

 Warning: One time out of ten after compiling the popup menu shows up
 the wrong place. Just recompile and it is ok again. I can't figure
 out why this is happening?
*/

#include <kapp.h>
#include <qpopmenu.h>
#include <qmlined.h>

#define COPY   1
#define PASTE  2

#ifndef _KAPP_H
#error Your app must be a KApplication and kapp.h must be included before klabel.h
#endif


class KMultiLineEdit : public QMultiLineEdit
{
   Q_OBJECT

public:
   KMultiLineEdit( QWidget *parent=0, const char *name=0 );
  ~KMultiLineEdit();

   QPopupMenu* getMenu() { return(menu); };

private:
   QPopupMenu *menu;
   QPoint     qp;
   const      QMouseEvent *mev;

signals:
   void   clicked();

public slots:
   void   slotCopy();
   void   slotPaste();
   void   slotDel();

protected:
   void   mousePressEvent( QMouseEvent * );
   void   keyPressEvent( QKeyEvent * );
   bool   eventFilter( QObject *, QEvent *);
   void   menuPopup();
};

#endif


