/*
 * KDE network utilities (knu.cpp)
 *
 * Bertrand Leconte <B.Leconte@mail.dotcom.fr>
 *
 * This program is free software (GPL). See the file COPYING for details.
 */

#include <kapp.h>
#include <ktabctl.h>

//#include <signal.h>
#include <sys/wait.h>
#include <unistd.h>

#include <kmsgbox.h>
#include <qfileinf.h>

#include "CommandDlg.h"
#include "TracerouteDlg.h"
#include "AboutDlg.h"
#include "HostDlg.h"

#include "knu.h"
#include "mknu.cpp"

// This is the first TopLevel object
TopLevel *toplevel;


/*
 * TopLevel::TopLevel - constructor
 * 
 */
TopLevel::TopLevel(QWidget *parent, const char *name, KConfig *kc)
    : QWidget(parent, name)
{
    setCaption("KDE network utilities");
    setMinimumSize(500, 300);

    kconfig = kc;

    /*
     * Create the tabctrl widget
     */
    tabCtrl = new KTabCtl(this);
    connect(tabCtrl, SIGNAL(tabSelected(int)), 
	    this, SLOT(slotTabChanged(int)));

    // ping tab
    CommandDlg *cd = new CommandDlg(tabCtrl);
    CHECK_PTR(cd);
    cd->setCommand("ping");
    tabCtrl->addTab(cd, "Ping");
    pages[0] = cd;

    // traceroute tab 
    TracerouteDlg *td = new TracerouteDlg(tabCtrl);
    CHECK_PTR(td);
    td->setCommand("traceroute");
    tabCtrl->addTab(td, "Traceroute");
    pages[1] = td;

    // host tab
    HostDlg *hd = new HostDlg(tabCtrl);
    CHECK_PTR(hd);
    hd->setCommand("host");
    tabCtrl->addTab(hd, "Host resolution");
    pages[2] = hd;
    
    
    // about tab
    QWidget *w = new AboutDlg(tabCtrl);
    CHECK_PTR(w);
    tabCtrl->addTab(w, "About");
    pages[3] = w;

    //tabCtrl->move(0, 0);
    currentTab = -1;

    // Create the buttons Help and Quit
    quitBtn = new QPushButton(this);
    quitBtn->setText("Quit");
    connect(quitBtn, SIGNAL(clicked()), SLOT(slotQuit()));
    quitBtn->resize(70, 30);
    
    helpBtn = new QPushButton(this);
    helpBtn->setText("Help");
    connect(helpBtn, SIGNAL(clicked()), SLOT(slotHelp()));
    helpBtn->resize(70, 30);

    // resize all this stuff
    adjustSize();
}

TopLevel::~TopLevel()
{
}

void 
TopLevel::resizeEvent(QResizeEvent *)
{
#define SEPARATION 10
  int buttonHeight;
  buttonHeight = quitBtn->height();
  
  tabCtrl->setGeometry(0, 0,
		       width(), height()-(2*SEPARATION + buttonHeight));
  quitBtn->move(width()-SEPARATION-quitBtn->width(),
		height() - buttonHeight - SEPARATION);
  helpBtn->move(SEPARATION,
		height() - buttonHeight - SEPARATION);
}


/*
 * called when a tab is changed
 */
void
TopLevel::slotTabChanged(int newTab)
{
  if (newTab != currentTab) {
    if ((currentTab>=0) && (pages[currentTab]->inherits("CommandDlg"))) {
      ((CommandDlg *)pages[currentTab])->tabDeselected();
    }
    if (pages[newTab]->inherits("CommandDlg")) {
      ((CommandDlg *)pages[newTab])->tabSelected();
    }
    currentTab = newTab;
  }
}


void 
TopLevel::slotQuit()
{
  //debug("TopLevel::slotQuit");
  delete toplevel;
  kapp->quit();
}

/*
 * This is called when the application is closed by the wm.
 * we make this like the quit button
 */
void 
TopLevel::closeEvent (QCloseEvent* qce)
{
  //debug("TopLevel::closeEvent");
  delete toplevel;
  QWidget::closeEvent(qce);
}

/*
 * help button
 * invokeHelp ???
 */
void 
TopLevel::slotHelp()
{
  QString   lfile, kfile, str;
  QFileInfo fi("knu.html");

  //debug("TopLevel::slotHelp");

  lfile = fi.absFilePath();
  if (!fi.exists()) { // first check current directory
    kfile = KApplication::kdedir();
    kfile += "/doc/HTML/" + fi.fileName();
    fi.setFile(kfile.data());
    if(!fi.exists()) { // then check HTML dir
      str =  "\nCouldn't find neither\n";
      str += lfile.data();
      str += "\nnor\n";
      str += kfile.data();
      KMsgBox::message(this, "Warning", str, 0, "Ok");
      return;
    }
    lfile = kfile;
  }
  if (fork() == 0) {
    execlp( "kdehelp", "kdehelp", lfile.data(), 0 );
    exit(1);
  }
}

/*
 * main - Application
 */
int
main(int argc, char **argv)
{
  const QString  appname = "knu";
  int            cs;
  KApplication  *a = new KApplication(argc, argv, appname);

  cs = a->getConfigState();
  toplevel = new TopLevel((QWidget*)0, (const char*) appname,
			  a->getConfig());
  CHECK_PTR(toplevel);

  a->setMainWidget(toplevel);
  toplevel->show();
  return a->exec();
}

