//
//  KPROCESSCONTROLLER -- A helper class for KProcess
//
//  version 0.0.1, July 13th 1997
//
//  (C) Christian Czezatke
//  e9025461@student.tuwien.ac.at
//

#include "kprocess.h"
#define _MAY_INCLUDE_KPROCESSCONTROLLER_
#include "kprocctrl.h"

#include "mkprocctrl.cpp"


KProcessController *theKProcessController = NULL;
static bool sigchld_occured = FALSE;

KProcessController::KProcessController()
{
 // initialize theKProcessList
 processList = new QList<KProcess>();
 CHECK_PTR(processList);
 
 timer_running =FALSE;
 timer = new QTimer();
 CHECK_PTR(timer);
 timer->changeInterval(timerCheckInterval);
 connect(timer, SIGNAL(timeout()), this, SLOT(slotDoHousekeeping()) );
 ::signal(SIGCHLD, theSigCHLDHandler);

}

void KProcessController::theSigCHLDHandler(int signal)
{
  int status;
  pid_t this_pid;
  bool found = FALSE;
  KProcess *proc;

  this_pid =  wait(&status);  // waitpid(-1, &this_status, WNOHANG);
 
  // now find a KProcess class for this Process...
  proc = theKProcessController->processList->first();
  while( (NULL != proc) && (!found) ) {
    found = (proc->pid == this_pid);
    if (!found)
      proc = theKProcessController->processList->next();
  }
  if (found) {
    proc->event_pending = TRUE;
    proc->status = status;
    sigchld_occured = TRUE;
  }
  ::signal(SIGCHLD, theSigCHLDHandler);
}


bool KProcessController::isTimerActive()
{
  return timer_running;
}

void KProcessController::startTimer()
{
  if(!timer_running) {
    timer_running = TRUE;
    timer->start(500);
  }
}

void KProcessController::stopTimer()
{
  if (timer_running) {
    timer->stop();
    timer_running = FALSE;
  }
}


void KProcessController::slotDoHousekeeping()
{
 bool still_need_timer = FALSE;
 KProcess *proc;
 
 if (!sigchld_occured) 
   return;
 
 // stop the timer -- if processing of all the 
 // terminating child processes takes longer than a timer
 // interval, we might get in trouble...
 sigchld_occured = FALSE;
  stopTimer();
 
 proc = processList->first();

 while (NULL != proc) {
   if (proc->event_pending) {
     // process has exited, so do emit the respective events
     proc->runs = FALSE;
     proc->event_pending = FALSE;
     if (notify_on_exit == proc->run_mode)
       proc->emitProcessExited();
   } else {
     // check whether there is still a process  running
     // in NOTIFY mode. If not, we can stop the timer
     if (!still_need_timer)
       still_need_timer = (proc->runs) && (proc->run_mode == notify_on_exit);
   }
   proc = processList->next();
 }
 if (still_need_timer)
   startTimer();
}

