#include "qpainter.h"
#include "qdrawutl.h"
#include "qfontmet.h"
#include "qpixmap.h"
#include "qkeycode.h"
#include "qclipbrd.h"
#include "qapp.h"

#include <ctype.h>
#include <kapp.h>
#include "mymlined.h"
#include "mmymlined.cpp"


MyMultiLineEdit::MyMultiLineEdit( QWidget *parent, const char *name )
    : QMultiLineEdit( parent, name )
{
    menu = new QPopupMenu((QWidget*)0, name);
    CHECK_PTR(menu);
    setReadOnly(TRUE);
    menu->insertItem( "Copy", this, SLOT(copyText()), CTRL+Key_Insert );
    //  menu->insertItem( "Paste", this, SLOT(paste()), SHIFT+Key_Insert );
    //menu->insertItem( "Cut", this, SLOT(cut()), SHIFT+Key_Delete );
    //menu->insertItem( "Delete", this, SLOT(slotDel()), Key_Delete );

    if ( parent )                           // has a child widget
    {
      menu->installEventFilter( this ); // then install filter
      installEventFilter( this );
    }
}

MyMultiLineEdit::~MyMultiLineEdit()
{
   ;
}

void MyMultiLineEdit::keyPressEvent( QKeyEvent *e ) 
{
    // convert ctrl-ins to ctrl-c
    if ( e->state() & ControlButton )
    {
       if(e->key() == Key_Home)
       {
         if ( e->state() & ShiftButton )
           setCursorPosition( 0, 0, TRUE);
         else
           setCursorPosition( 0, 0, FALSE);
         return;
       }
       if(e->key() == Key_End)
       {
         bool state;
         if ( e->state() & ShiftButton )
           state = TRUE;
         else
         state = FALSE;
         while(!atEnd())
         {
           cursorDown(state);
           end(state);
         }
       }
       if(e->key() == Key_Insert)
       {
         QKeyEvent ke(Event_KeyPress, Key_C, 0, ControlButton);
         QMultiLineEdit::keyPressEvent( &ke );
         return;
       }
    } // convert shift-insert to ctrl-v
    else if ( e->state() & ShiftButton )
    {
       if(e->key() == Key_Insert)
       {
         QKeyEvent ke(Event_KeyPress, Key_V, 0, ControlButton);
         QMultiLineEdit::keyPressEvent( &ke );
         return;
       }
    } // if its neither we just pass it on
    QMultiLineEdit::keyPressEvent( e );
}

/*!
  Emits the \e clicked() signal so that we can catch it in a slot.
*/
void   MyMultiLineEdit::mousePressEvent( QMouseEvent *me )
{
    qp = QCursor::pos();

    // save the event for the popup menu
    mev = me;
    emit clicked();

    // we have to send on the event in order to let QLineEdit
    // handle it but if we send it with a RMB event we can't
    // do mouse actions on selected text
    if(me->button() != RightButton)
      QMultiLineEdit::mousePressEvent( me );
    else
      menuPopup();
};

bool MyMultiLineEdit::eventFilter( QObject *ob, QEvent *e )
{
    QPoint p = QCursor::pos();

    // prevent popupmenu from disappearing when button is released
    if (ob == menu && e->type() == Event_MouseButtonRelease
	&& p == qp )
    {
	qp = QPoint(-10,-10);
        QMouseEvent *me = (QMouseEvent*)e;
        if(me->button() == RightButton)
 	  return TRUE;
    }
    return FALSE;                           // standard event processing
}

void   MyMultiLineEdit::menuPopup( )
{
    QWidget *mw = kapp->mainWidget();
    QRect mrect = mw->geometry(), rect = geometry();
    QPoint qpoint = mev->pos();

    // the position we get is relative to the label but the popup is
    // relative to the screen so we must adjust it
    qpoint += mrect.topLeft();
    qpoint += rect.topLeft();

    menu->popup(qpoint, 0);
}

// We use the accelerators already defined
void MyMultiLineEdit::slotCopy()
{
    QKeyEvent ke(Event_KeyPress, Key_C, 0, ControlButton);
    QMultiLineEdit::keyPressEvent( &ke );
}

void MyMultiLineEdit::slotPaste()
{
    QKeyEvent ke(Event_KeyPress, Key_V, 0, ControlButton);
    QMultiLineEdit::keyPressEvent( &ke );
}

// this should remove the marked text but not touch the clipboard.
// it also put the marked text in the clipboard but I wont change
// it cause it will propably be changed in new version of qt.
void   MyMultiLineEdit::slotDel()
{
    QKeyEvent ke(Event_KeyPress, Key_Delete, 0, 0);
    QMultiLineEdit::keyPressEvent( &ke );
}



void 
MyMultiLineEdit::insertAt(const char *txt, int line, int col)
{
  // Workaround for Qt 1.2 bug.
  // Remove this when 1.2 is no longer supported
  // (from Paul Olav Tvete <paul@troll.no>)
  line = QMAX(QMIN(line, numLines() - 1), 0);
  setAutoUpdate(FALSE);
  insertLine("", -1);
  QMultiLineEdit::insertAt(txt, line, col);
  if (numLines()>1) {
    removeLine(numLines() - 1);
  }
  setAutoUpdate(TRUE);
  repaint();
}
